package codechicken.enderstorage.storage.item;

import net.minecraft.client.renderer.Tessellator;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.client.renderer.tileentity.TileEntityRenderer;
import net.minecraft.client.renderer.tileentity.TileEntitySpecialRenderer;
import net.minecraftforge.client.ForgeHooksClient;

import org.lwjgl.opengl.GL11;
import org.lwjgl.opengl.GL12;
import org.lwjgl.util.vector.Matrix4f;
import org.lwjgl.util.vector.Vector3f;

import codechicken.core.ClientUtils;
import codechicken.core.render.CCModelLibrary;
import codechicken.core.render.CCRenderState;
import codechicken.core.render.RenderCustomEndPortal;
import codechicken.core.vec.Matrix4;
import codechicken.core.vec.Quat;
import codechicken.core.vec.Vector3;
import codechicken.enderstorage.api.EnderStorageManager;
import codechicken.enderstorage.internal.EnderStorageClientProxy;
import codechicken.enderstorage.storage.item.TileEnderChest;

import static codechicken.core.render.RenderUtils.*;

public class EnderChestRenderer extends TileEntitySpecialRenderer
{
	private static ModelEnderChest model = new ModelEnderChest();
    
    public EnderChestRenderer()
    {
    }

    public static void renderChest(int rotation, int freq, boolean owned, double x, double y, double z, int offset, float lidAngle)
    {
        TileEntityRenderer info = TileEntityRenderer.instance;
        renderEndPortal.render(x, y, z, 0, info.playerX, info.playerY, info.playerZ, info.renderEngine);
        GL11.glColor4f(1, 1, 1, 1);
        
        ForgeHooksClient.bindTexture("/codechicken/enderstorage/gfx/enderchest.png", 0);        
        GL11.glPushMatrix();
        GL11.glEnable(GL12.GL_RESCALE_NORMAL);
        GL11.glColor4f(1.0F, 1.0F, 1.0F, 1.0F);
        GL11.glTranslated(x, y + 1.0, z + 1.0F);
        GL11.glScalef(1.0F, -1F, -1F);
        GL11.glTranslatef(0.5F, 0.5F, 0.5F);
        GL11.glRotatef(rotation*90, 0.0F, 1.0F, 0.0F);
        GL11.glTranslatef(-0.5F, -0.5F, -0.5F);
        model.chestLid.rotateAngleX = lidAngle;
        model.render(owned);
        GL11.glPopMatrix();
        
        GL11.glPushMatrix();
        GL11.glTranslated(x, y, z);
        renderButtons(freq, rotation, lidAngle);
        GL11.glPopMatrix();
        
        double time = ClientUtils.getRenderTime()+offset;
        Matrix4 pearlMat = new Matrix4()
            .translate(new Vector3(x+0.5, y+0.2+lidAngle*-0.5+EnderStorageClientProxy.getPearlBob(time), z+0.5))
            .scale(new Vector3(0.04, 0.04, 0.04))
            .rotate(time/3, new Vector3(0, 1, 0));
        
        GL11.glDisable(GL11.GL_LIGHTING);
        CCRenderState.changeTexture("/codechicken/enderstorage/gfx/hedronmap.png");
        CCRenderState.startDrawing(4);
        CCModelLibrary.icosahedron4.render(pearlMat, 0, 0);
        CCRenderState.draw();
        GL11.glEnable(GL11.GL_LIGHTING);
    }

	private static void renderButtons(int freq, int rot, double lidAngle)
	{
		ForgeHooksClient.bindTexture("/codechicken/enderstorage/gfx/buttons.png", 0);
				
		drawButton(0, EnderStorageManager.getColourFromFreq(freq, 0), rot, lidAngle);
		drawButton(1, EnderStorageManager.getColourFromFreq(freq, 1), rot, lidAngle);
		drawButton(2, EnderStorageManager.getColourFromFreq(freq, 2), rot, lidAngle);
	}

	private static void drawButton(int button, int colour, int rot, double lidAngle)
	{
		float texx = 0.25F * (float)(colour % 4);
		float texy = 0.25F * (float)(colour / 4);
		
        GL11.glPushMatrix();
		
        EnderDyeButton ebutton = TileEnderChest.buttons[button].copy();
        ebutton.rotate(0, 0.5625, 0.0625, 1, 0, 0, lidAngle);
        ebutton.rotateMeta(rot);
		Vector3[] verts = ebutton.verts;
		
		Tessellator tessellator = Tessellator.instance;
		tessellator.startDrawingQuads();
		addVecWithUV(verts[7], texx+0.0938, texy+0.0625);
		addVecWithUV(verts[3], texx+0.0938, texy+0.1875);
		addVecWithUV(verts[2], texx+0.1562, texy+0.1875);
		addVecWithUV(verts[6], texx+0.1562, texy+0.0625);

		addVecWithUV(verts[4], texx+0.0938, texy+0.0313);
		addVecWithUV(verts[7], texx+0.0938, texy+0.0313);
		addVecWithUV(verts[6], texx+0.1562, texy+0.0624);
		addVecWithUV(verts[5], texx+0.1562, texy+0.0624);
		
		addVecWithUV(verts[0], texx+0.0938, texy+0.2186);
		addVecWithUV(verts[1], texx+0.1562, texy+0.2186);
		addVecWithUV(verts[2], texx+0.1562, texy+0.1876);
		addVecWithUV(verts[3], texx+0.0938, texy+0.1876);

		addVecWithUV(verts[6], texx+0.1563, texy+0.0626);
		addVecWithUV(verts[2], texx+0.1563, texy+0.1874);
		addVecWithUV(verts[1], texx+0.1874, texy+0.1874);
		addVecWithUV(verts[5], texx+0.1874, texy+0.0626);

		addVecWithUV(verts[7], texx+0.0937, texy+0.0626);
		addVecWithUV(verts[4], texx+0.0626, texy+0.0626);
		addVecWithUV(verts[0], texx+0.0626, texy+0.1874);
		addVecWithUV(verts[3], texx+0.0937, texy+0.1874);
		tessellator.draw();
		
		GL11.glPopMatrix();
	}

    public void renderTileEntityAt(TileEntity tile, double d, double d1, double d2, float f)
    {
        CCRenderState.reset();
        CCRenderState.setBrightness(tile.worldObj, tile.xCoord, tile.yCoord, tile.zCoord);
        CCRenderState.useNormals(true);
        
        TileEnderChest chest = (TileEnderChest)tile;
        renderChest(chest.rotation, chest.freq, !chest.owner.equals("global"), d, d1, d2, 
                EnderStorageClientProxy.getTimeOffset(chest.xCoord, chest.yCoord, chest.zCoord), (float) chest.getRadianLidAngle(f));
    }  
    
    public static final double phi = 1.618034;
    
    static RenderCustomEndPortal renderEndPortal = new RenderCustomEndPortal(0.626, 0.188, 0.812, 0.188, 0.812);
}
